function [] = servoCmd(robot, commandType, value)
%Sends a command to our EX-106+ servo controller
%Commandtype:
%0      - reset angles
%1-3    - set joint 1-3 to given value
%4      - ask for angles of all 3 attached joints
%5      - torque on
%6      - torque off
%7      - set PID values. Value should contain the target joint
%8      - set max speed in degrees per second. Value should contain the target joint

    if (commandType >= 1 && commandType <= 3) %Set angle
        %Clamp the value to the range (-120, 120).
        %The robot does this as well, but this is repeated here as too large
        %a value*256 converted to 2 bytes can overflow into a smaller angle
        %causing undesired behaviour.
        if value > 120
            value = 120;
        elseif value < -120
           value = -120; 
        end
    elseif (commandType == 7) || (commandType == 8)
        %PID set OR max speed set
        if (value < 1) || (value > 3) 
            return;
        end
    end

    %Send header + packet
    byteValues = typecast(int16(round(value*256)), 'int8');
    toSend = [int8(commandType), byteValues(1), byteValues(2)]; %joint1, lsb, msb
    fwrite(robot.ctrl, toSend, 'int8');

    %PID change needs additional values
    if commandType == 7
       %Send the actual PID values
       byteValues = typecast(int16(round(robot.P*256)), 'int8'); %P
       toSend = [byteValues(1), byteValues(2)]; %lsb, msb
       fwrite(robot.ctrl, toSend, 'int8');
       byteValues = typecast(int16(round(robot.I*256)), 'int8'); %I
       toSend = [byteValues(1), byteValues(2)]; %lsb, msb
       fwrite(robot.ctrl, toSend, 'int8');
       byteValues = typecast(int16(round(robot.D*256)), 'int8'); %D
       toSend = [byteValues(1), byteValues(2)]; %lsb, msb
       fwrite(robot.ctrl, toSend, 'int8');
    elseif commandType == 8
       %Send the desired max speed in degrees per second
       byteValues = typecast(int16(round(robot.speed*256)), 'int8'); %P
       toSend = [byteValues(1), byteValues(2)]; %lsb, msb
       fwrite(robot.ctrl, toSend, 'int8');
    end

end

