#include "X2Joint.h"

#define JOINT_LOW 1
#define JOINT_HIGH 3

//NOTE: Angles larger than 120 are percieved as smaller

//const int D3 = 2; //If high write to RS485 bus, if low listen
const int ledPin = 13;
//const double PI = 3.1415;


double angle = 0;
int angle_i;
int iteration = 0;
int ledState = LOW;

double angle_read = 0;
double angle_read_last = 0;

double tmp_P, tmp_I, tmp_D;

unsigned long time_prev;

//X2Joint joint1(1, 7, 0.8, 5); //id=1, PID=2,5,1
//X2Joint joint1(1, 40, 0.9, 1); //id=1, PID=2,5,1

X2Joint joint1(1, 40, 4, 5); //base
X2Joint joint2(2, 7, 40, 6); //shoulder
X2Joint joint3(3, 6, 60, 6); //wrist

X2Joint *joints[3] = {&joint1, &joint2, &joint3};  

void setup() {
  Serial.begin(115200);
  joints[0]->beginComm(); //open serial for all joints
  for (int i = JOINT_LOW; i < JOINT_HIGH; i++) {
    //Prepare and reset all joints
    joints[i]->begin();
  }
  pinMode(13, OUTPUT);

  delay(500);
  while(Serial.available()){
    Serial.read();
  }

  time_prev = 0;
}


double parseDouble() {
  //Fetches 2 bytes from the serial, converts it to a double
  double val;
  uint8_t buf[2];
  while(Serial.available() < 2); //Wait for data
  Serial.readBytes((char*)buf, 2);

  val = (double)((buf[1]<<8) | buf[0]);
  val = val / ((double) 256);
  return val;
}


void loop() {
  unsigned long time_now = millis();

  if(Serial.available() > 2) {
    //At least one full packet has arrived. Decode it:
    uint8_t buf[3];
    Serial.readBytes((char*)buf, 3);
   
    //Decode packets
    switch(buf[0]) {
      case 0:
        //Reset all joint's angles
        for (int i = JOINT_LOW; i < JOINT_HIGH; i++) {
          joints[i]->begin();
        }
        break;
      case 1: //joint 1
      case 2: //joint 2
      case 3: //joint 3
        angle = (double)((buf[2]<<8) | buf[1]);
        angle = angle / ((double) 256);
      
        //Joint set to 'angle'
        joints[buf[0]-1]->setAngle(angle);
        break;
      
      case 4:
        //Reply with current joint angles
        for (int i = 0; i < 3; i++) {
          angle = joints[i]->getAngle()*256;
          //Serial.println(joints[i]->getAngle());
          angle_i = (int)angle;
          Serial.write(angle_i);
          Serial.write(angle_i >> 8);
        }
        
        break;      
      case 5:
        //Motor torque on
        for (int i = 0; i < 3; i++) {
          joints[i]->torqueOn();
        }
        break;      
      case 6:
        //Motor torque off, also disables update of PID values to prevent surprises when turned back on
        for (int i = 0; i < 3; i++) {
          joints[i]->torqueOff();
        }        
        break;
      case 7:
        //Set PID values
        tmp_P = parseDouble();
        tmp_I = parseDouble();
        tmp_D = parseDouble();
        Serial.print("Joint ");
        Serial.print(buf[2]);
        Serial.print(", ");
        Serial.print(tmp_P);
        Serial.print(", ");
        Serial.print(tmp_I);
        Serial.print(", ");
        Serial.println(tmp_D);
        joints[buf[2]-1]->setPID(tmp_P, tmp_I, tmp_D);
        break;
      case 8:
        //Set max speed. Convert received degrees per second to deg per 10ms
        joints[buf[2]-1]->setMaxspeed(parseDouble()/100.0);
        break;          
      default:
      
        break;
      
      }
  }

  for (int i = JOINT_LOW; i < JOINT_HIGH; i++) {
    //Query the servos and run the PID loop of each joint
    //joints[i]->update(i == 1 ? (int)(sin((joints[i]->getAngle()*PI)/180.0)*-300.0) : 0);
    joints[i]->update(i == 1 ? (int)(sin((joints[i]->getAngle()*PI)/180.0)*-250.0) : 0);
    //joints[i]->update(0);
  }
  

  iteration++;
  if (iteration > 250) {
    iteration = 0;
    digitalWrite(ledPin, ledState);
    ledState ^= 1;
  }
  
 /* while(time_now < (time_prev+1));
  time_prev = time_now;*/
}
