#ifndef X2Joint_h
#define X2Joint_h
#include "Arduino.h"
#include <PID_v1.h>

#define DEGREES_PER_ENC 0.06                           //Degrees of servo movement per step of the encoder
#define DEGREES_PER_STEP ((DEGREES_PER_ENC*15)/64)     //Degrees of joint movement per step of the encoder. 15 teeth on servo, 64 on joint.
#define CLOCKWISE 0
#define COUNTERCLOCKWISE 1

class X2Joint {
 public:
	X2Joint(int id, int PID_P, int PID_I, int PID_D);
     void begin();
	 void beginComm();

     /* Methods for moving around and printing to the buffer */
	void setAngle(double angle);
     void setPID(double P, double I, double D);
     double getAngle();
     void update(int angleCompensation);
     int query();
     void torqueOff();
     void torqueOn();
     void setTorque(uint16_t value, uint8_t direction);
     void addChecksum(uint8_t* arr, uint8_t len);
     int readReg2(uint8_t regLSB);
     void writeReg2(uint8_t regLSB, uint16_t value);
     void writeReg1(uint8_t regNo, uint8_t value);
     void PID(int angleCompensation);
     void speedLimiter(unsigned long time_curr);
     void setMaxspeed(double angleper10ms);

     uint8_t torqueEnable; //0 disables PID calulcation,

 private:
     double angle_curr;        //Current angle
     double angle_target;      //Target angle
     double torque;            //Torque output of PID loop
     int encoder_prev;         //Previous encoder value read from servo
     uint8_t servo_id;
     uint8_t rw_pin; //This pin selects whether we write(1) to the MAX485 or reads(0) from it

     double PID_ITerm, PID_angle_last;
     double kp;                  // * (P)roportional Tuning Parameter
     double ki;                  // * (I)ntegral Tuning Parameter
     double kd;                  // * (D)erivative Tuning Parameter

     uint8_t error;           //Set to 1 if readreg2 fails. 0 otherwise
     unsigned long time_prev; //The time in millis when we calculated the angle previously
     double spd_angle_maxspeed;
     double spd_angle_sum;
     double spd_correction;
     double spd_kp, spd_ki;
     double spd_ITerm;
};

#endif
